==> nipper-0.10.4.tgz: nipper-0.10.4/CSS/process-access-list.c <==
/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#define css_permit 0
#define css_deny 1
#define css_bypass 2

// Access List Permission
struct cssAclClause
{
	int number;
	int access;								// css_permit, css_deny or css_bypass
	int log;								// true or false (default)
	char protocol[8];
	char source[32];
	char sourcePort[8];
	char sourcePortEnd[8];
	int sourcePortOper;						// perm_op_lt, perm_op_gt...
	char dest[32];
	char destPort[8];
	char destPortEnd[8];
	int destPortOper;						// perm_op_lt, perm_op_gt...
	struct cssAclClause *next;
};


// Access List
struct cssAcl
{
	int index;								// The ACL index no.
	char applyTo[64];						// Where the ACL is applied to
	struct cssAclClause *clause;			// ACL permission
	struct cssAcl *next;
};


// Process Access List
void processCSSACL(char *line, struct nipperConfig *nipper)
{
	// Variables...
	int tempInt;
	int setting;
	fpos_t filePosition;
	char tempString[nipper->maxSize];
	struct cssAclClause *cssAclClausePointer;
	struct cssAcl *cssAclPointer;
	struct ciscoCommand command;

	// Debug
	if (nipper->debugMode == true)
	{
		printf("Access-List Line: %s\n", line);
	}

	// Init
	command = splitLine(line);
	if (strcmp(command.part[0], "no") == 0)
	{
		setting = false;
		tempInt = 2;
	}
	else
	{
		setting = true;
		tempInt = 1;
	}

	// If enabled ...
	if (strcmp(command.part[tempInt], "enable") == 0)
		nipper->css->aclEnabled = true;

	// If Disabled
	else if(strcmp(command.part[tempInt], "disable") == 0)
		nipper->css->aclEnabled = false;

	// Else it is a new ACL
	else
	{
		// Create ACL struct
		if (nipper->css->acl == 0)
		{
			nipper->css->acl = malloc(sizeof(struct cssAcl));
			cssAclPointer = nipper->css->acl;
		}
		else
		{
			cssAclPointer = nipper->css->acl;
			while (cssAclPointer->next != 0)
				cssAclPointer = cssAclPointer->next;
			cssAclPointer->next = malloc(sizeof(struct cssAcl));
			cssAclPointer = cssAclPointer->next;
		}

		// Init Structure
		memset(cssAclPointer, 0, sizeof(struct cssAcl));

		// Index
		cssAclPointer->index = atoi(command.part[tempInt]);

		// Get the rules...
		fgetpos(nipper->input, &filePosition);
		readLine(nipper->input, line, nipper->maxSize);
		while ((feof(nipper->input) == 0) && (command.parts != 0))
		{
			// Debug
			if (nipper->debugMode == true)
			{
				printf("Access-List Line: %s\n", line);
			}

			// Init
			command = splitLine(line);

			// Apply
			if (strcmp(command.part[0], "apply") == 0)
				strncpy(cssAclPointer->applyTo, command.part[1], sizeof(cssAclPointer->applyTo) -1);

			// Clause
			else if (strcmp(command.part[0], "clause") == 0)
			{
				tempInt = 1;

				// Create clause struct
				if (cssAclPointer->clause == 0)
				{
					cssAclPointer->clause = malloc(sizeof(struct cssAclClause));
					cssAclClausePointer = cssAclPointer->clause;
				}
				else
				{
					cssAclClausePointer = cssAclPointer->clause;
					while (cssAclClausePointer->next != 0)
						cssAclClausePointer = cssAclClausePointer->next;
					cssAclClausePointer->next = malloc(sizeof(struct cssAclClause));
					cssAclClausePointer = cssAclClausePointer->next;
				}

				// Init Structure
				memset(cssAclClausePointer, 0, sizeof(struct cssAcl));
				strcpy(cssAclClausePointer->sourcePort, "any");
				cssAclClausePointer->sourcePortOper = perm_op_eq;
				strcpy(cssAclClausePointer->destPort, "any");
				cssAclClausePointer->destPortOper = perm_op_eq;

				// Clause no.
				cssAclClausePointer->number = atoi(command.part[tempInt]);
				tempInt++;

				// Log?
				if (strcmp(command.part[tempInt], "log") == 0)
				{
					tempInt++;
					if (strcmp(command.part[tempInt], "enable") == 0)
						cssAclClausePointer->log = true;
					else
						cssAclClausePointer->log = false;
					tempInt++;
				}
				else
					cssAclClausePointer->log = false;

				// Bypass / permit / deny
				if (strcmp(command.part[tempInt], "bypass") == 0)
					cssAclClausePointer->access = css_bypass;
				else if (strcmp(command.part[tempInt], "permit") == 0)
					cssAclClausePointer->access = css_permit;
				else
					cssAclClausePointer->access = css_deny;
				tempInt++;

				// Protocol
				strncpy(cssAclClausePointer->protocol, command.part[tempInt], sizeof(cssAclClausePointer->protocol) -1);
				tempInt++;

				// Source
				strcpy(tempString, command.part[tempInt]);
				tempInt++;
				if ((strcmp(command.part[tempInt], "destination") != 0) && (strcmp(command.part[tempInt], "range") != 0) && (strcmp(command.part[tempInt], "neq") != 0) && (strcmp(command.part[tempInt], "gt") != 0) && (strcmp(command.part[tempInt], "lt") != 0) && (strcmp(command.part[tempInt], "eq") != 0))
				{
					sprintf(tempString, "%s %s", tempString, command.part[tempInt]);
					tempInt++;
				}
				strncpy(cssAclClausePointer->source, tempString, sizeof(cssAclClausePointer->source) -1);

				// Source port
				if (strcmp(command.part[tempInt], "destination") != 0)
				{
					// Equal...
					if (strcmp(command.part[tempInt], "eq") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						cssAclClausePointer->sourcePortOper = perm_op_eq;
					}

					// Less than
					else if (strcmp(command.part[tempInt], "lt") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						cssAclClausePointer->sourcePortOper = perm_op_lt;
					}

					// Greater than
					else if (strcmp(command.part[tempInt], "gt") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						cssAclClausePointer->sourcePortOper = perm_op_gt;
					}

					// Not equal to...
					else if (strcmp(command.part[tempInt], "neq") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						cssAclClausePointer->sourcePortOper = perm_op_neq;
					}

					// Range
					else if (strcmp(command.part[tempInt], "range") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						tempInt++;
						strncpy(cssAclClausePointer->sourcePortEnd, command.part[tempInt], sizeof(cssAclClausePointer->sourcePortEnd) -1);
						cssAclClausePointer->sourcePortOper = perm_op_range;
					}
					tempInt++;
				}

				// destination
				tempInt++;
				strcpy(tempString, command.part[tempInt]);
				tempInt++;
				if ((tempInt < command.parts) && (strcmp(command.part[tempInt], "prefer") != 0) && (strcmp(command.part[tempInt], "sourcegroup") != 0) && (strcmp(command.part[tempInt], "range") != 0) && (strcmp(command.part[tempInt], "neq") != 0) && (strcmp(command.part[tempInt], "gt") != 0) && (strcmp(command.part[tempInt], "lt") != 0) && (strcmp(command.part[tempInt], "eq") != 0))
				{
					sprintf(tempString, "%s %s", tempString, command.part[tempInt]);
					tempInt++;
				}
				strncpy(cssAclClausePointer->dest, tempString, sizeof(cssAclClausePointer->source) -1);

				// Destination port
				if ((strcmp(command.part[tempInt], "range") == 0) || (strcmp(command.part[tempInt], "neq") == 0) || (strcmp(command.part[tempInt], "gt") == 0) || (strcmp(command.part[tempInt], "lt") == 0) || (strcmp(command.part[tempInt], "eq") == 0))
				{
					// Equal...
					if (strcmp(command.part[tempInt], "eq") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						cssAclClausePointer->destPortOper = perm_op_eq;
					}

					// Less than
					else if (strcmp(command.part[tempInt], "lt") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						cssAclClausePointer->destPortOper = perm_op_lt;
					}

					// Greater than
					else if (strcmp(command.part[tempInt], "gt") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						cssAclClausePointer->destPortOper = perm_op_gt;
					}

					// Not equal to...
					else if (strcmp(command.part[tempInt], "neq") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						cssAclClausePointer->destPortOper = perm_op_neq;
					}

					// Range
					else if (strcmp(command.part[tempInt], "range") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						tempInt++;
						strncpy(cssAclClausePointer->destPortEnd, command.part[tempInt], sizeof(cssAclClausePointer->destPortEnd) -1);
						cssAclClausePointer->destPortOper = perm_op_range;
					}
				}
			}

			// Get next
			fgetpos(nipper->input, &filePosition);
			readLine(nipper->input, line, nipper->maxSize);
		}

		// Set to previous position...
		fsetpos(nipper->input, &filePosition);
	}
}

==> nipper-0.10.5.tgz: nipper-0.10.5/CSS/process-access-list.c <==
/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#define css_permit 0
#define css_deny 1
#define css_bypass 2

// Access List Permission
struct cssAclClause
{
	int number;
	int access;								// css_permit, css_deny or css_bypass
	int log;								// true or false (default)
	char protocol[8];
	char source[32];
	char sourcePort[8];
	char sourcePortEnd[8];
	int sourcePortOper;						// perm_op_lt, perm_op_gt...
	char dest[32];
	char destPort[8];
	char destPortEnd[8];
	int destPortOper;						// perm_op_lt, perm_op_gt...
	struct cssAclClause *next;
};


// Access List
struct cssAcl
{
	int index;								// The ACL index no.
	char applyTo[64];						// Where the ACL is applied to
	struct cssAclClause *clause;			// ACL permission
	struct cssAcl *next;
};


// Process Access List
void processCSSACL(char *line, struct nipperConfig *nipper)
{
	// Variables...
	int tempInt = 0;
	int setting = 0;
	fpos_t filePosition;
	char tempString[nipper->maxSize];
	struct cssAclClause *cssAclClausePointer = 0;
	struct cssAcl *cssAclPointer = 0;
	struct ciscoCommand command;

	// Debug
	if (nipper->debugMode == true)
	{
		printf("Access-List Line: %s\n", line);
	}

	// Init
	command = splitLine(line);
	if (strcmp(command.part[0], "no") == 0)
	{
		setting = false;
		tempInt = 2;
	}
	else
	{
		setting = true;
		tempInt = 1;
	}

	// If enabled ...
	if (strcmp(command.part[tempInt], "enable") == 0)
		nipper->css->aclEnabled = true;

	// If Disabled
	else if(strcmp(command.part[tempInt], "disable") == 0)
		nipper->css->aclEnabled = false;

	// Else it is a new ACL
	else
	{
		// Create ACL struct
		if (nipper->css->acl == 0)
		{
			nipper->css->acl = malloc(sizeof(struct cssAcl));
			cssAclPointer = nipper->css->acl;
		}
		else
		{
			cssAclPointer = nipper->css->acl;
			while (cssAclPointer->next != 0)
				cssAclPointer = cssAclPointer->next;
			cssAclPointer->next = malloc(sizeof(struct cssAcl));
			cssAclPointer = cssAclPointer->next;
		}

		// Init Structure
		memset(cssAclPointer, 0, sizeof(struct cssAcl));

		// Index
		cssAclPointer->index = atoi(command.part[tempInt]);

		// Get the rules...
		fgetpos(nipper->input, &filePosition);
		readLine(nipper->input, line, nipper->maxSize);
		while ((feof(nipper->input) == 0) && (command.parts != 0))
		{
			// Debug
			if (nipper->debugMode == true)
			{
				printf("Access-List Line: %s\n", line);
			}

			// Init
			command = splitLine(line);

			// Apply
			if (strcmp(command.part[0], "apply") == 0)
				strncpy(cssAclPointer->applyTo, command.part[1], sizeof(cssAclPointer->applyTo) -1);

			// Clause
			else if (strcmp(command.part[0], "clause") == 0)
			{
				tempInt = 1;

				// Create clause struct
				if (cssAclPointer->clause == 0)
				{
					cssAclPointer->clause = malloc(sizeof(struct cssAclClause));
					cssAclClausePointer = cssAclPointer->clause;
				}
				else
				{
					cssAclClausePointer = cssAclPointer->clause;
					while (cssAclClausePointer->next != 0)
						cssAclClausePointer = cssAclClausePointer->next;
					cssAclClausePointer->next = malloc(sizeof(struct cssAclClause));
					cssAclClausePointer = cssAclClausePointer->next;
				}

				// Init Structure
				memset(cssAclClausePointer, 0, sizeof(struct cssAcl));
				strcpy(cssAclClausePointer->sourcePort, "any");
				cssAclClausePointer->sourcePortOper = perm_op_eq;
				strcpy(cssAclClausePointer->destPort, "any");
				cssAclClausePointer->destPortOper = perm_op_eq;

				// Clause no.
				cssAclClausePointer->number = atoi(command.part[tempInt]);
				tempInt++;

				// Log?
				if (strcmp(command.part[tempInt], "log") == 0)
				{
					tempInt++;
					if (strcmp(command.part[tempInt], "enable") == 0)
						cssAclClausePointer->log = true;
					else
						cssAclClausePointer->log = false;
					tempInt++;
				}
				else
					cssAclClausePointer->log = false;

				// Bypass / permit / deny
				if (strcmp(command.part[tempInt], "bypass") == 0)
					cssAclClausePointer->access = css_bypass;
				else if (strcmp(command.part[tempInt], "permit") == 0)
					cssAclClausePointer->access = css_permit;
				else
					cssAclClausePointer->access = css_deny;
				tempInt++;

				// Protocol
				strncpy(cssAclClausePointer->protocol, command.part[tempInt], sizeof(cssAclClausePointer->protocol) -1);
				tempInt++;

				// Source
				strcpy(tempString, command.part[tempInt]);
				tempInt++;
				if ((strcmp(command.part[tempInt], "destination") != 0) && (strcmp(command.part[tempInt], "range") != 0) && (strcmp(command.part[tempInt], "neq") != 0) && (strcmp(command.part[tempInt], "gt") != 0) && (strcmp(command.part[tempInt], "lt") != 0) && (strcmp(command.part[tempInt], "eq") != 0))
				{
					sprintf(tempString, "%s %s", tempString, command.part[tempInt]);
					tempInt++;
				}
				strncpy(cssAclClausePointer->source, tempString, sizeof(cssAclClausePointer->source) -1);

				// Source port
				if (strcmp(command.part[tempInt], "destination") != 0)
				{
					// Equal...
					if (strcmp(command.part[tempInt], "eq") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						cssAclClausePointer->sourcePortOper = perm_op_eq;
					}

					// Less than
					else if (strcmp(command.part[tempInt], "lt") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						cssAclClausePointer->sourcePortOper = perm_op_lt;
					}

					// Greater than
					else if (strcmp(command.part[tempInt], "gt") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						cssAclClausePointer->sourcePortOper = perm_op_gt;
					}

					// Not equal to...
					else if (strcmp(command.part[tempInt], "neq") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						cssAclClausePointer->sourcePortOper = perm_op_neq;
					}

					// Range
					else if (strcmp(command.part[tempInt], "range") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->sourcePort, command.part[tempInt], sizeof(cssAclClausePointer->sourcePort) -1);
						tempInt++;
						strncpy(cssAclClausePointer->sourcePortEnd, command.part[tempInt], sizeof(cssAclClausePointer->sourcePortEnd) -1);
						cssAclClausePointer->sourcePortOper = perm_op_range;
					}
					tempInt++;
				}

				// destination
				tempInt++;
				strcpy(tempString, command.part[tempInt]);
				tempInt++;
				if ((tempInt < command.parts) && (strcmp(command.part[tempInt], "prefer") != 0) && (strcmp(command.part[tempInt], "sourcegroup") != 0) && (strcmp(command.part[tempInt], "range") != 0) && (strcmp(command.part[tempInt], "neq") != 0) && (strcmp(command.part[tempInt], "gt") != 0) && (strcmp(command.part[tempInt], "lt") != 0) && (strcmp(command.part[tempInt], "eq") != 0))
				{
					sprintf(tempString, "%s %s", tempString, command.part[tempInt]);
					tempInt++;
				}
				strncpy(cssAclClausePointer->dest, tempString, sizeof(cssAclClausePointer->source) -1);

				// Destination port
				if ((strcmp(command.part[tempInt], "range") == 0) || (strcmp(command.part[tempInt], "neq") == 0) || (strcmp(command.part[tempInt], "gt") == 0) || (strcmp(command.part[tempInt], "lt") == 0) || (strcmp(command.part[tempInt], "eq") == 0))
				{
					// Equal...
					if (strcmp(command.part[tempInt], "eq") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						cssAclClausePointer->destPortOper = perm_op_eq;
					}

					// Less than
					else if (strcmp(command.part[tempInt], "lt") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						cssAclClausePointer->destPortOper = perm_op_lt;
					}

					// Greater than
					else if (strcmp(command.part[tempInt], "gt") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						cssAclClausePointer->destPortOper = perm_op_gt;
					}

					// Not equal to...
					else if (strcmp(command.part[tempInt], "neq") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						cssAclClausePointer->destPortOper = perm_op_neq;
					}

					// Range
					else if (strcmp(command.part[tempInt], "range") == 0)
					{
						tempInt++;
						strncpy(cssAclClausePointer->destPort, command.part[tempInt], sizeof(cssAclClausePointer->destPort) -1);
						tempInt++;
						strncpy(cssAclClausePointer->destPortEnd, command.part[tempInt], sizeof(cssAclClausePointer->destPortEnd) -1);
						cssAclClausePointer->destPortOper = perm_op_range;
					}
				}
			}

			// Get next
			fgetpos(nipper->input, &filePosition);
			readLine(nipper->input, line, nipper->maxSize);
		}

		// Set to previous position...
		fsetpos(nipper->input, &filePosition);
	}
}
